// src/CreatePlugin.ts
import {
  getPathWhereIdentifier as getPathWhereIdentifier2
} from "@magnetarjs/types";
import { filterDataPerClauses as filterDataPerClauses2 } from "@magnetarjs/utils";
import { copy } from "copy-anything";
import { pick } from "filter-anything";
import { mapGetOrSet } from "getorset-anything";
import { isArray as isArray2, isNumber as isNumber3, isPlainObject as isPlainObject2 } from "is-what";

// src/actions/delete.ts
import { isFullString } from "is-what";
function deleteActionFactory(data, vue2StoreOptions, makeBackup) {
  return function({
    payload,
    collectionPath,
    docId,
    pluginModuleConfig
  }) {
    const { vueInstance: vue } = vue2StoreOptions;
    const _docId = docId || payload;
    if (!isFullString(_docId))
      throw new Error("No ID passed to delete action.");
    if (makeBackup)
      makeBackup(collectionPath, _docId);
    vue.delete(data[collectionPath], _docId);
  };
}

// src/actions/deleteProp.ts
import { isArray } from "is-what";
import { getProp } from "path-to-prop";
function deletePropActionFactory(data, vue2StoreOptions, makeBackup) {
  return function({
    payload,
    collectionPath,
    docId,
    pluginModuleConfig
  }) {
    const { vueInstance: vue } = vue2StoreOptions;
    if (!docId)
      throw new Error("An non-existent action was triggered on a collection");
    const collectionDic = data[collectionPath];
    const docData = collectionDic[docId];
    if (!docData)
      throw new Error(`Document data not found for id: ${collectionPath} ${docId}`);
    if (makeBackup)
      makeBackup(collectionPath, docId);
    const payloadArray = isArray(payload) ? payload : [payload];
    for (const propToDelete of payloadArray) {
      const isNestedPropPath = /[./]/.test(propToDelete);
      if (isNestedPropPath) {
        const parts = propToDelete.split(/[./]/);
        const lastPart = parts.pop();
        const parentRef = getProp(docData, parts.join("."));
        vue.delete(parentRef, lastPart || "");
      } else {
        vue.delete(docData, propToDelete);
      }
    }
  };
}

// src/actions/fetch.ts
import { filterDataPerClauses } from "@magnetarjs/utils";
import { isBoolean, isFullString as isFullString3, isNumber as isNumber2 } from "is-what";

// src/actions/insert.ts
import { isFullString as isFullString2, isNumber } from "is-what";
function insertActionFactory(data, vue2StoreOptions, makeBackup) {
  const { generateRandomId, vueInstance: vue } = vue2StoreOptions;
  return function({
    payload,
    collectionPath,
    docId,
    pluginModuleConfig
  }) {
    const collectionDic = data[collectionPath];
    const _docId = docId || (isFullString2(payload.id) || isNumber(payload.id) ? String(payload.id) : generateRandomId());
    if (makeBackup)
      makeBackup(collectionPath, _docId);
    const docDataToMutate = collectionDic[_docId];
    if (!docDataToMutate) {
      vue.set(collectionDic, _docId, payload);
      return _docId;
    }
    Object.entries(payload).forEach(([key, value]) => {
      vue.set(docDataToMutate, key, value);
    });
    return _docId;
  };
}

// src/actions/fetch.ts
function fetchActionFactory(data, exists, vue2StoreOptions) {
  return function({
    payload,
    collectionPath,
    docId,
    actionConfig,
    pluginModuleConfig
  }) {
    const force = payload?.force === true;
    const optimisticFetch = !force;
    if (optimisticFetch) {
      if (!docId) {
        const { query, where, orderBy, limit } = pluginModuleConfig;
        const collectionData = filterDataPerClauses(new Map(Object.entries(data[collectionPath])), {
          query,
          where,
          orderBy,
          limit
        });
        if (Object.keys(collectionData).length > 0) {
          const localDocs = Object.entries(collectionData).map(([_docId, data2]) => ({
            data: data2,
            exists: "unknown",
            id: _docId
          }));
          const fetchResponse = { docs: localDocs };
          return fetchResponse;
        }
      }
      const itSureExists = exists[`${collectionPath}/${docId}`] === true;
      if (docId && itSureExists) {
        const localDoc = data[collectionPath][docId];
        if (localDoc) {
          const fetchResponse = {
            docs: [
              {
                data: localDoc,
                exists: "unknown",
                id: docId
              }
            ]
          };
          return fetchResponse;
        }
        if (!localDoc) {
        }
      }
    }
    const doOnFetchAction = (_payload, meta) => {
      const _docId = docId ? docId : meta === "error" ? void 0 : isFullString3(meta?.id) ? meta.id : isNumber2(meta?.id) ? `${meta.id}` : void 0;
      const docPath = `${collectionPath}/${_docId}`;
      if (meta === "error") {
        exists[docPath] = "error";
        return;
      }
      if (isBoolean(meta?.exists)) {
        exists[docPath] = meta.exists;
      }
      if (_payload === void 0)
        return;
      insertActionFactory(
        data,
        vue2StoreOptions
      )({
        payload: _payload,
        collectionPath,
        docId: _docId,
        actionConfig,
        pluginModuleConfig
      });
      return _payload;
    };
    return doOnFetchAction;
  };
}

// src/actions/fetchCount.ts
import {
  getPathWhereIdentifier
} from "@magnetarjs/types";
function fetchCountActionFactory(pathCountDic, vue2StoreOptions) {
  return function({
    collectionPath,
    actionConfig,
    pluginModuleConfig
  }) {
    const pathId = getPathWhereIdentifier(collectionPath, pluginModuleConfig);
    const doOnFetchCountAction = ({ count }) => {
      if (count === void 0)
        return;
      pathCountDic[pathId] = count;
    };
    return doOnFetchCountAction;
  };
}

// src/actions/mergeAssignReplace.ts
import { merge } from "merge-anything";
import { isPlainObject } from "is-what";
function writeActionFactory(data, vue2StoreOptions, actionName, makeBackup) {
  const { vueInstance: vue } = vue2StoreOptions;
  return function({
    payload,
    collectionPath,
    docId,
    pluginModuleConfig
  }) {
    if (!docId)
      throw new Error("An non-existent action was triggered on a collection");
    const collectionDic = data[collectionPath];
    if (makeBackup)
      makeBackup(collectionPath, docId);
    if (actionName === "replace" || !collectionDic[docId]) {
      vue.set(collectionDic, docId, {});
    }
    const docDataToMutate = collectionDic[docId];
    if (!docDataToMutate)
      throw new Error(`Document data not found for id: ${collectionPath} ${docId}`);
    if (actionName === "merge") {
      Object.entries(payload).forEach(([key, value]) => {
        const originalValue = docDataToMutate[key];
        if (isPlainObject(originalValue) && isPlainObject(value)) {
          vue.set(docDataToMutate, key, merge(originalValue, value));
        } else {
          vue.set(docDataToMutate, key, value);
        }
      });
    }
    if (actionName === "assign" || actionName === "replace") {
      Object.entries(payload).forEach(([key, value]) => {
        vue.set(docDataToMutate, key, value);
      });
    }
  };
}

// src/actions/revert.ts
function revertActionFactory(data, vue2StoreOptions, restoreBackup) {
  return function({
    payload,
    collectionPath,
    docId,
    pluginModuleConfig,
    actionName,
    error
  }) {
    if (actionName === "stream" || actionName === "fetch") {
      return;
    }
    if (docId && ["insert", "merge", "assign", "replace", "delete", "deleteProp"].includes(actionName)) {
      restoreBackup(collectionPath, docId);
      return;
    }
    if (!docId && actionName === "insert")
      actionName = "insert on collections";
    console.error(`[@magnetarjs/plugin-vue2] revert not yet implemented for ${actionName}`);
  };
}

// src/actions/stream.ts
import { isFullString as isFullString4, isString } from "is-what";
function streamActionFactory(data, vue2StoreOptions) {
  return function({
    payload,
    collectionPath,
    docId,
    actionConfig,
    pluginModuleConfig,
    mustExecuteOnRead
  }) {
    const doOnStream = {
      added: (payload2, meta) => {
        if (payload2 === void 0)
          return;
        const _docId = docId || `${meta.id}`;
        insertActionFactory(
          data,
          vue2StoreOptions
        )({
          payload: payload2,
          collectionPath,
          docId: _docId,
          actionConfig,
          pluginModuleConfig
        });
      },
      modified: (payload2, meta) => {
        if (payload2 === void 0)
          return;
        const _docId = docId || `${meta.id}`;
        insertActionFactory(
          data,
          vue2StoreOptions
        )({
          payload: payload2,
          collectionPath,
          docId: _docId,
          actionConfig,
          pluginModuleConfig
        });
      },
      removed: (payload2, meta) => {
        if (payload2 === void 0)
          return;
        const collectionPathDocIdToDelete = isFullString4(docId) ? [collectionPath, docId] : isString(payload2) ? [collectionPath, payload2] : [collectionPath, meta.id];
        const [_cPath, _dId] = collectionPathDocIdToDelete;
        deleteActionFactory(
          data,
          vue2StoreOptions
        )({
          payload: void 0,
          collectionPath: _cPath,
          docId: _dId,
          actionConfig,
          pluginModuleConfig
        });
      }
    };
    return doOnStream;
  };
}

// src/helpers/dataHelpers.ts
function objectToMap(object, originalObjectToClear, entriesInCustomOrder) {
  const dic = object || {};
  function get(id) {
    return dic[id];
  }
  function has(id) {
    return !!dic[id];
  }
  function keys() {
    if (entriesInCustomOrder)
      return entriesInCustomOrder.map((e) => e[0]);
    return Object.keys(dic);
  }
  function values() {
    if (entriesInCustomOrder)
      return entriesInCustomOrder.map((e) => e[1]);
    return Object.values(dic);
  }
  function entries() {
    if (entriesInCustomOrder)
      return entriesInCustomOrder;
    return Object.entries(dic);
  }
  function clear() {
    if (!originalObjectToClear)
      return;
    if (entriesInCustomOrder) {
      entriesInCustomOrder.forEach(([key]) => {
        delete originalObjectToClear[key];
      });
    } else {
      Object.keys(originalObjectToClear).forEach((key) => {
        delete originalObjectToClear[key];
      });
    }
  }
  function _delete(id) {
    const existed = id in dic;
    if (existed)
      delete dic[id];
    return existed;
  }
  function forEach(callbackfn) {
    const _entries = entriesInCustomOrder || Object.entries(dic);
    _entries.forEach(([k, v]) => {
      callbackfn(v, k, /* @__PURE__ */ new Map());
    });
  }
  const raw = dic;
  const customMap = {
    size: Object.keys(dic).length,
    get,
    has,
    keys,
    values,
    entries,
    forEach,
    clear,
    delete: _delete,
    raw
  };
  return customMap;
}

// src/CreatePlugin.ts
var CreatePlugin = (vue2StoreOptions) => {
  const { vueInstance: vue } = vue2StoreOptions;
  const data = vue.observable({});
  const exists = vue.observable({});
  const pathCountDic = vue.observable({});
  const dataBackups = {};
  const makeBackup = (collectionPath, docId) => {
    if (!(collectionPath in dataBackups))
      dataBackups[collectionPath] = /* @__PURE__ */ new Map();
    const backupCollectionMap = dataBackups[collectionPath];
    const arr = mapGetOrSet(backupCollectionMap, docId, () => []);
    const foundDoc = data[collectionPath][docId];
    if (foundDoc)
      arr.push(copy(foundDoc));
  };
  const restoreBackup = (collectionPath, docId) => {
    if (!(collectionPath in dataBackups))
      return;
    const backupCollectionMap = dataBackups[collectionPath];
    if (!backupCollectionMap.has(docId))
      return;
    const docBackupArray = backupCollectionMap.get(docId);
    if (!docBackupArray || !docBackupArray.length) {
      vue.delete(data[collectionPath], docId);
      return;
    }
    const docBackup = docBackupArray.pop();
    if (docBackup)
      vue.set(data[collectionPath], docId, docBackup);
    if (docBackup === void 0)
      vue.delete(data[collectionPath], docId);
  };
  const modulesAlreadySetup = /* @__PURE__ */ new Set();
  const setupModule = ({
    collectionPath,
    docId,
    pluginModuleConfig = {}
  }) => {
    const modulePath = [collectionPath, docId].filter(Boolean).join("/");
    if (modulesAlreadySetup.has(modulePath))
      return;
    if (!(collectionPath in data))
      vue.set(data, collectionPath, {});
    const dataCollectionDic = data[collectionPath];
    modulesAlreadySetup.add(modulePath);
    const { initialData } = pluginModuleConfig;
    if (!initialData)
      return;
    if (!docId && isArray2(initialData)) {
      if (Object.keys(dataCollectionDic).length > 0)
        return;
      for (const [_docId, _docData] of initialData) {
        vue.set(dataCollectionDic, _docId, _docData);
      }
    } else if (docId && isPlainObject2(initialData)) {
      if (docId in dataCollectionDic)
        return;
      vue.set(dataCollectionDic, docId, initialData);
    }
  };
  const getModuleData = ({
    collectionPath,
    docId,
    pluginModuleConfig = {}
  }) => {
    const dataCollectionDic = data[collectionPath];
    if (docId)
      return dataCollectionDic[docId];
    const clauses = pick(pluginModuleConfig, [
      "query",
      "where",
      "orderBy",
      "limit",
      "startAfter"
    ]);
    const result = filterDataPerClauses2(new Map(Object.entries(dataCollectionDic)), clauses);
    const resultEntries = [...result.entries()];
    const resultAsDic = Object.fromEntries(resultEntries);
    return objectToMap(resultAsDic, dataCollectionDic, resultEntries);
  };
  const getModuleExists = ({
    collectionPath,
    docId
  }) => {
    return exists[`${collectionPath}/${docId}`];
  };
  const getModuleCount = ({
    collectionPath,
    pluginModuleConfig = {}
  }) => {
    const pathId = getPathWhereIdentifier2(collectionPath, pluginModuleConfig);
    const count = pathCountDic[pathId];
    if (isNumber3(count))
      return count;
    const clauses = pick(pluginModuleConfig, [
      "query",
      "where",
      "orderBy",
      "limit",
      "startAfter"
    ]);
    const collectionDB = data[collectionPath];
    const dataFiltered = filterDataPerClauses2(new Map(Object.entries(collectionDB)), clauses);
    return dataFiltered.size;
  };
  const fetch = fetchActionFactory(data, exists, vue2StoreOptions);
  const fetchCount = fetchCountActionFactory(pathCountDic, vue2StoreOptions);
  const stream = streamActionFactory(data, vue2StoreOptions);
  const insert = insertActionFactory(data, vue2StoreOptions, makeBackup);
  const _merge = writeActionFactory(data, vue2StoreOptions, "merge", makeBackup);
  const assign = writeActionFactory(data, vue2StoreOptions, "assign", makeBackup);
  const replace = writeActionFactory(data, vue2StoreOptions, "replace", makeBackup);
  const deleteProp = deletePropActionFactory(data, vue2StoreOptions, makeBackup);
  const _delete = deleteActionFactory(data, vue2StoreOptions, makeBackup);
  const revert = revertActionFactory(data, vue2StoreOptions, restoreBackup);
  const instance = {
    revert,
    actions: {
      fetch,
      fetchCount,
      stream,
      insert,
      merge: _merge,
      assign,
      replace,
      deleteProp,
      delete: _delete
    },
    setupModule,
    getModuleData,
    getModuleExists,
    getModuleCount
  };
  return instance;
};
export {
  CreatePlugin
};
